/**************************************************************************************************
 *  @file ABZB_APP_TASK.c.
 *
 *  This file contains common procedures used in the application
 *

 *
 *  Copyright 2004-2005. Airbee Wireless Inc. All rights reserved.
 *	This program is protected by US and International copyright laws.

 *	Warning: This program is protected by copyright law and international
 *	treaties. Unauthorized reproduction or distribution of this program, or
 *	any portion of it, may result in severe civil or criminal penalties, and
 *	will be prosecuted to the maximum extent possible under the law.*
 *
 *
 *	Date		Author			Modifications		Version
 *  01 Jul 05	        Airbee		          NONE		          1.0
 *																
 *
* ***************************************** Header File Inclusion ********************************/

#include "ABZB_HAL_Platform.h"     // Header file for selection for Hardware platform
#include "ABZB_FOUR_NODE_APP.h"
#include "stdlib.h"                     //Standard Library used for Dynamic memory allocation

UINT8 G_ABZB_Uch_Devices_joined;        //which has a count of no. of devices joined it

/***********************************************************************************************
Description:	
Entry point of the APP_TASK , which the task is registered in the kernel
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       -

Called from     :	Called when the kernel switches to this task i.e., any message is posted in the APP_MBX

CAUTION         :       THIS FUNCTION NEEDS NO CHANGE !!!

***********************************************************************************************/
__task void V_ABZB_APP_TASK_Main()
{
        V_ABZB_APP_DeviceConfig0();                                 //Selecting the device as in the preprocessor which in turn will assign the IEEE address
        V_ABZB_ASP_APP_SWITCH_LED_DemoInit();                       //Making the LED and port init on MSP430

#ifdef CO_ORDINATOR
        V_ABZB_APP_COORDINATOR_INIT();                              //initialisations for the coordinator
        V_ABZB_ZADKV2_Binding_Table_config();                       // Called to configure the Binding table
#else
       V_ABZB_APP_ROUTER_INIT();                                    //init for the ROUTER config
#endif
       V_ABZB_APP_Forever_Loop();
} // end of V_ABZB_APP_TASK_Main function.
/***********************************************************************************************/


/***********************************************************************************************
Description:	
Loop that exists forever till there is a valid message in the APP_MBX
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       -

Called from     :	Called when the kernel switches to this task i.e., any message is posted in the APP_MBX

Caution         :       User can handlie his code here
                        "free" for the alloted memory (by the NWK layer on sending confirm/indications) should be taken care ..
                        All the confirmations and indication messages posted in the APP mailbox by the NWK layer are not handled.

***********************************************************************************************/
void V_ABZB_APP_Forever_Loop()
{
  tMACMsgDetails    Msg={0};                                      //structure variable which will hold the MSG id and pointer from the kernel
  while(TRUE)
  {
        GET_MESSAGE(APP_MBX, &(Msg.Msg));                         //getting message from the Application mailbox
        switch(Msg.MACMsg.MsgId)                                  //switches to a case depending on the message ID
        {
    	    case NLDE_DATA_INDICATION:                            //Indicates that there is data indication from the NWK layer
            {
                UINT8 *ABZB_Uch_Ptr;                                //declaring an unsigned character pointer
                ABZB_Uch_Ptr = (UINT8*)(((stNLDEDATAIndication*)(Msg.MACMsg.Message.MsgPtr))->Nsdu);  //Getting the NSDU from the data indication pointer
                switch(*ABZB_Uch_Ptr)                               //protocol implemantation : first byte of the NSDU is the protocol ID
                {
                  case M_ABZB_PROTOCOL_ID_PERMIT_JOIN:
                      V_ABZB_APP_Send_NLME_PermitJoin(M_ABZB_PERMIT_DURATION);  //Sending the permit join request to the NWK layer on reception of this protocol ID
                      break;

                  case M_ABZB_PROTOCOL_ID_APP_DATA:
                      V_ABZB_ASP_SLC_OnOffSRC(M_ABZB_LED1);                      //If the rceived is the APPLICATION data toggle the LED1 by callong the SRC
                      break;
                  default:
                  {}
                }
                free(((stNLDEDATAIndication *)(Msg.MACMsg.Message.MsgPtr))->Nsdu);  //"free" the NSDU data pointer
                break;
    	    }
	    	
	   case NLME_JOIN_INDICATION:
	      G_ABZB_Uch_Devices_joined++;                    //increment the no. of devices joined on receipt of the join indication
	      #ifdef CO_ORDINATOR
	      if(G_ABZB_Uch_Devices_joined == M_ABZB_ALLOWABLE_DEVICES_2) //checking whether 2 devices joined to PAN/C in this case
	      {
        	      G_ABZB_Uch_Devices_joined = ZERO;
        	      free(Msg.MACMsg.Message.MsgPtr);            //"free" the message pointer sent by the NWK layer
        	      V_ABZB_APP_Send_PermitEnable_To_Router();   //if so then disable its own permit duration and send a data protocol packet to ROUTER to enable the permit duration
              }
              #else                                           //applicable for ROUTER
              G_ABZB_Uch_Devices_joined = ZERO;               //restotre the variable to ZERO
              free(Msg.MACMsg.Message.MsgPtr);                //"free" the message pointer
              V_ABZB_APP_Send_NLME_PermitJoin(ZERO);          //in case of ROUTER only one device will join the network in this case hence disable the
              #endif                                          //hence disbale permit duration if one device joined the network
              break;        	
	
	   case M_DEBOUNCE_EVENT:                           //the message is posted on mailbox when a keypress is occured hence construct a data packet and send it ti NWK
               V_ABZB_APP_Send_NLDEDataRequest();
               break;

           case M_ABZB_APP_FRMN_JOIN_FAILURE:
               V_ABZB_ASP_SLC_OnOffSRC(M_ABZB_LED2);         //If the rceived is the JOIN CONFIRM toggle the LED2 by callong the SRC
               V_ABZB_Timer_Soft_set_timer3(M_ABZB_APP_FRMN_JOIN_FAILURE, M_ABZB_APP_LED_FLASH_DURATION, APP_MBX);         //again post the error in the MBX so that LED will blink
               break;
            default:
            {}
          }//SWITCH
          free(Msg.MACMsg.Message.MsgPtr);                  //"free" the message pointer
        }//while(1)
}


/***********************************************************************************************
Description:	
Constructs a RESET REQUEST packet and send it to the NWK layer, Binding information is also initialised
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       -

Called from     :	called from V_ABZB_APP_COORDINATOR_INIT() if CO_ORDINATOR is defined and V_ABZB_APP_ROUTER_INIT() if CO_ORDINATOR is not defined

Caution         :       "free" the message pointer on receipt of the RESET CONFIRM message sent by the NWK layer
***********************************************************************************************/
void V_ABZB_APP_NetworkResetRequest0()
{
     tMACMsgDetails    Msg={0};
     V_ABZB_NWK_NLMERESETRequest();                   //SENDING NLME-RESET REQUEST
     while(Msg.MACMsg.MsgId != NLME_RESET_CONFIRM)    //waiting till the NLME_RESET_CONFIRM is received
     GET_MESSAGE(APP_MBX, &(Msg.Msg));                //Getting message from the APP mailbox
     free(Msg.MACMsg.Message.MsgPtr);                 //"free" the message pointer sent by the NWK layer
     V_ABZB_ZADKV2_Init_Bind_Info_Table();            // Called to initialize the binding Table i.e., all the SOURCE ADDRESS will be initialised
}


/***********************************************************************************************
Description:	
             Constructs a NETWORK FORMATION REQUEST packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       -

Called from     :	called from V_ABZB_APP_COORDINATOR_INIT()

Caution         :       Applicable for PAN Coordinator only
                        "free" the message pointer on receipt of the NETWORK FORMATION CONFIRM message sent by the NWK layer
***********************************************************************************************/

void V_ABZB_APP_NetworkFormationRequest0()
{
     tMACMsgDetails    Msg = {0};
     stNLMENWKFORMATIONRequest *NLME_NWK_Formation_Request;      //declaring a structure pointer to give the Network formation request
     NLME_NWK_Formation_Request = (stNLMENWKFORMATIONRequest*)malloc(sizeof(stNLMENWKFORMATIONRequest));

     if(NLME_NWK_Formation_Request != NULL_PTR)                 //Checking whether pointer is not NULL
     {
       NLME_NWK_Formation_Request->ScanChannels         = M_ABZB_SCAN_CHANNELS;       //Scan channels
       NLME_NWK_Formation_Request->PANId                = M_ABZB_PANID;               //panid;
       NLME_NWK_Formation_Request->ScanDuration         = M_ABZB_SCAN_DURATION;       //scan duration
       NLME_NWK_Formation_Request->BeaconOrder          = M_ABZB_BEACONORDER;         //non-beacon mode
       NLME_NWK_Formation_Request->SuperframeOrder      = M_ABZB_SUPERFAMEORDER;
       NLME_NWK_Formation_Request->BatteryLifeExtension = FALSE;

       V_ABZB_NWK_NLMENETWORKFORMATIONRequest(NLME_NWK_Formation_Request);            //sending the NLME Formation Request

       while(Msg.MACMsg.MsgId != NLME_NETWORK_FORMATION_CONFIRM)                      //waiting till the NLME_NETWORK_FORMATION_CONFIRM is received
       GET_MESSAGE(APP_MBX, &(Msg.Msg));                                              //getting message from the APP mailbox
       if(((stNLMENWKFORMATIONConfirm *)(Msg.MACMsg.Message.MsgPtr))->Status == ZERO)   //checking whether the confirm status is success (0 for success)
       {
             V_ABZB_ASP_SLC_OnOffSRC(M_ABZB_LED2);                                      //If the rceived is the FORMATION CONFIRM toggle the LED2 by callong the SRC
       }
       else
       {
            V_ABZB_Timer_Soft_set_timer3(M_ABZB_APP_FRMN_JOIN_FAILURE, M_ABZB_APP_LED_FLASH_DURATION, APP_MBX);
       }                                                                          //do nothing
       free(Msg.MACMsg.Message.MsgPtr);                                               //"free" the message pointer sent by the NWK layer
     }
}

/***********************************************************************************************
Description:	
             Constructs a PERMIT JOIN REQUEST packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       Permit duration either 0x00 or 0xFF

Called from     :	called from V_ABZB_APP_COORDINATOR_INIT() in case of CO_ORDINATOR and in case of ROUTER ,on receipt
                        DATA packet from the PAN/C with the protocol ID (0x04).

Caution         :       Applicable for PAN Coordinator only
                        "free" the message pointer on receipt of the PERMIT JOIN CONFIRM message sent by the NWK layer
***********************************************************************************************/
void V_ABZB_APP_Send_NLME_PermitJoin(UINT8 ABZB_Uch_PermitDuration)
{
    tMACMsgDetails    Msg= {0};
    stNLMEPERMITJOININGRequest *NLME_Permit_Joining_Request;              //declaring a structure pointer
    NLME_Permit_Joining_Request = (stNLMEPERMITJOININGRequest*)malloc(sizeof(stNLMEPERMITJOININGRequest));
    if(NLME_Permit_Joining_Request != NULL_PTR)                           //checking whether the pointer is not NULL
    {
        NLME_Permit_Joining_Request->PermitDuration = ABZB_Uch_PermitDuration;  //Assigning the permit duration , received as a parameter
        V_ABZB_NWK_NLMEPERMITJOININGRequest(NLME_Permit_Joining_Request);       //sending the NLME permit join request to the NWK layer

       while(Msg.MACMsg.MsgId != NLME_PERMIT_JOINING_CONFIRM)                   //waiting till the permit join confim is received
       GET_MESSAGE(APP_MBX, &(Msg.Msg));                                        //getting message from the APP mailbox
       free(Msg.MACMsg.Message.MsgPtr);                                         //"free" the message pointer sent by the NWL layer
    }
}


/***********************************************************************************************
Description:	
             Constructs a DISCOVERY REQUEST packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from V_ABZB_APP_ROUTER_INIT()

Caution         :       Applicable for ROUTER only
                        "free" the message pointer on receipt of the DISCOVERY CONFIRM message sent by the NWK layer
***********************************************************************************************/

void V_ABZB_APP_NetworkDiscoveryRequest0()
{
    tMACMsgDetails    Msg ={0};                                   //Declaring a instant named "Msg" of type tMACMsgDetails
    stNLMENWKDISCOVERYRequest *NLME_NWK_DISCOVERY_Request;        //declaring the structure pointer of type stNLMENWKDISCOVERYRequest
    NLME_NWK_DISCOVERY_Request =  (stNLMENWKDISCOVERYRequest*)malloc(sizeof(stNLMENWKDISCOVERYRequest));

    if(NLME_NWK_DISCOVERY_Request != NULL_PTR)                              //checking whether the pointer is NULL
    {
      NLME_NWK_DISCOVERY_Request->ScanChannels = M_ABZB_SCAN_CHANNELS;      //Giving the scan channels
      NLME_NWK_DISCOVERY_Request->ScanDuration = M_ABZB_SCAN_DURATION;      //scan duration
      V_ABZB_NWK_NLMENETWORKDISCOVERYRequest(NLME_NWK_DISCOVERY_Request);   //sending the NLME Discovery request

      while(Msg.MACMsg.MsgId != NLME_NETWORK_DISCOVERY_CONFIRM)              //waiting till the Netwok discovery confirm is received.
      GET_MESSAGE(APP_MBX, &(Msg.Msg));                                      //getting the message from the APP mailbox
      free(Msg.MACMsg.Message.MsgPtr);                                       //free the message pointer sent by the NWK layer
    }
}

/***********************************************************************************************
Description:	
             Constructs a JOIN REQUEST packet and send it to the NWK layer.On JOIN confirm message, if it is not
             SUCCESS , then constructs a rejoin request packet and send it to the NWK layer.
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from V_ABZB_APP_ROUTER_INIT()

Caution         :       Applicable for ROUTER only
                        "free" the message pointer on receipt of the DISCOVERY CONFIRM message sent by the NWK layer
***********************************************************************************************/

void V_ABZB_APP_NetworkJoinRequest0()
{
    tMACMsgDetails    Msg ={0};                     //Declaring a instant named "Msg" of type tMACMsgDetails
    stNLMEJOINRequest *NLME_Join_Request ;          //declaring the structure pointer of type stNLMEJOINRequest
    NLME_Join_Request = (stNLMEJOINRequest *)malloc(sizeof(stNLMEJOINRequest ));

    if(NLME_Join_Request != NULL_PTR)               //checking whether the pointer is NULL
    {
      NLME_Join_Request->ScanChannels   = M_ABZB_SCAN_CHANNELS;   //SCAN CHANNELS
      NLME_Join_Request->PANId          = M_ABZB_PANID;           //PAN ID
      NLME_Join_Request->ScanDuration   = M_ABZB_SCAN_DURATION;   //SCAN DURATION
      NLME_Join_Request->PowerSource    = ZERO;
      NLME_Join_Request->RxOnWhenIdle   = TRUE;
      NLME_Join_Request->MACSecurity    = ZERO;
      NLME_Join_Request->JoinAsRouter   = TRUE;
      NLME_Join_Request->RejoinNetwork  = FALSE;

      V_ABZB_NWK_NLMEJOINRequest(NLME_Join_Request);                      //Sending the NLME Join Request

      while(Msg.MACMsg.MsgId != NLME_JOIN_CONFIRM)                        //waiting till the NLME_JOIN_CONFIRM is receined
      GET_MESSAGE(APP_MBX, &(Msg.Msg));                                   //Getting the message from the APP mailbox
      if(((stNLMEJOINConfirm *)(Msg.MACMsg.Message.MsgPtr))->Status == ZERO)  //checking whether the confirm status is success
      {
            V_ABZB_ASP_SLC_OnOffSRC(M_ABZB_LED2);                      //If the rceived is the JOIN CONFIRM toggle the LED2 by callong the SRC
      }
      else                                                              //else post the error ID in the APP_MABX so that LED will blink
      {
          //below is for orphan, i.e., Rejoin the NWK
          NLME_Join_Request = (stNLMEJOINRequest *)malloc(sizeof(stNLMEJOINRequest ));

          if(NLME_Join_Request != NULL_PTR)               //checking whether the pointer is NULL
          {
            NLME_Join_Request->ScanChannels   = M_ABZB_SCAN_CHANNELS;   //SCAN CHANNELS
            NLME_Join_Request->PANId          = M_ABZB_PANID;           //PAN ID
            NLME_Join_Request->ScanDuration   = M_ABZB_SCAN_DURATION;   //SCAN DURATION
            NLME_Join_Request->PowerSource    = ZERO;
            NLME_Join_Request->RxOnWhenIdle   = TRUE;
            NLME_Join_Request->MACSecurity    = ZERO;
            NLME_Join_Request->JoinAsRouter   = TRUE;
            NLME_Join_Request->RejoinNetwork  = TRUE;

            V_ABZB_NWK_NLMEJOINRequest(NLME_Join_Request);                      //Sending the NLME Join Request

            while(Msg.MACMsg.MsgId != NLME_JOIN_CONFIRM)                        //waiting till the NLME_JOIN_CONFIRM is receined
            GET_MESSAGE(APP_MBX, &(Msg.Msg));
            //teh abvove confirm will give just a orphan scan confirm

            //clring the message ID as the next confirm nneds to be get from APP MAIL BOX again
            Msg.MACMsg.MsgId             = CLEAR;

            while(Msg.MACMsg.MsgId != NLME_JOIN_CONFIRM)                        //waiting till the NLME_JOIN_CONFIRM is receined
            GET_MESSAGE(APP_MBX, &(Msg.Msg));

            if(((stNLMEJOINConfirm *)(Msg.MACMsg.Message.MsgPtr))->Status == ZERO)  //checking whether the confirm status is success
            {
                  V_ABZB_ASP_SLC_OnOffSRC(M_ABZB_LED2);                      //If the rceived is the JOIN CONFIRM toggle the LED2 by callong the SRC
            }
            else
            {
                  V_ABZB_Timer_Soft_set_timer3(M_ABZB_APP_FRMN_JOIN_FAILURE, M_ABZB_APP_LED_FLASH_DURATION, APP_MBX);
            }

          }                                                            //else do nothing
          free(Msg.MACMsg.Message.MsgPtr);
        }                                             //"free" the message pointer sent by the NWK layer
    }
}

/***********************************************************************************************
Description:	
             Constructs a START ROUTER REQUEST packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Typ;e     :	void

Parameter Passed:       none

Called from     :	called from V_ABZB_APP_ROUTER_INIT()

Caution         :       Applicable for ROUTER only
                        "free" the message pointer on receipt of the DISCOVERY CONFIRM message sent by the NWK layer
***********************************************************************************************/

void V_ABZB_APP_Network_StartRouterRequest0(void)
{
    tMACMsgDetails    Msg= {0};                             //Declaring a instant named "Msg" of type tMACMsgDetails
    stNLMESTARTROUTERRequest *NLME_Start_Router_Request;    //declaring the structure pointer of type stNLMESTARTROUTERRequest
    NLME_Start_Router_Request = (stNLMESTARTROUTERRequest*)malloc(sizeof(stNLMESTARTROUTERRequest));

    if(NLME_Start_Router_Request != NULL_PTR)               //checking whether the pointer is NULL
    {
      NLME_Start_Router_Request->BeaconOrder          = M_ABZB_BEACONORDER;   //Beacon order (non-beacon mode)
      NLME_Start_Router_Request->SuperFrameOrder      = M_ABZB_SUPERFAMEORDER;//superframe order
      NLME_Start_Router_Request->BatteryLifeExtension = FALSE;

      V_ABZB_NWK_NLMESTARTROUTERRequest(NLME_Start_Router_Request);   //Sending the Start router request

      while(Msg.MACMsg.MsgId != NLME_START_ROUTER_CONFIRM)            //waiting till the START ROUTER CONFIRM IS RECEIVED
      GET_MESSAGE(APP_MBX, &(Msg.Msg));                               //getting the message from the APP mailbox
      free(Msg.MACMsg.Message.MsgPtr);                                //"free" the message pointer sent by the NWK layer
    }
}

/***********************************************************************************************
Description:	
             Selects the IEEE address for the device depending on the preprocessor definition chosen in ABZB_APP_TASK.h
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called on entering the V_ABZB_APP_TASK_Main()

***********************************************************************************************/
void V_ABZB_APP_DeviceConfig0()
{
#ifdef CO_ORDINATOR
    V_ABZB_ZADKV2_Select_Device(M_ABZB_ZADKV2_CO_ORDINATOR);    //selecting the device as COORDINATOR, which in turn the IEEE address will be assigned
#elif defined ROUTER1
    V_ABZB_ZADKV2_Select_Device(M_ABZB_ZADKV2_ROUTER1);         //selecting the device as ROUTER1, which in turn the IEEE address will be assigned
#elif defined ROUTER2
    V_ABZB_ZADKV2_Select_Device(M_ABZB_ZADKV2_ROUTER2);         //selecting the device as ROUTER2, which in turn the IEEE address will be assigned
#elif defined ROUTER3
    V_ABZB_ZADKV2_Select_Device(M_ABZB_ZADKV2_ROUTER3);         //selecting the device as ROUTER3, which in turn the IEEE address will be assigned
#endif
}

/***********************************************************************************************
Description:	
             Selects the SOurce and destination network address in the network for the data to be transmitted
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_TASK_Main()

CAUTION         :       applicable for PAN/C only
***********************************************************************************************/

void V_ABZB_ZADKV2_Binding_Table_config()
{
     V_ABZB_ZADKV2_Update_Bind_Table( M_ABZB_ZADKV2_CO_ORDINATOR, M_ABZB_ZADKV2_ROUTER3 );
     V_ABZB_ZADKV2_Update_Bind_Table( M_ABZB_ZADKV2_ROUTER3, M_ABZB_ZADKV2_CO_ORDINATOR );
     V_ABZB_ZADKV2_Update_Bind_Table( M_ABZB_ZADKV2_ROUTER1, M_ABZB_ZADKV2_ROUTER2 );
     V_ABZB_ZADKV2_Update_Bind_Table( M_ABZB_ZADKV2_ROUTER2, M_ABZB_ZADKV2_ROUTER1 );
}

/***********************************************************************************************
Description:	
             Make the necessary APIs to init the hardware (LED and DIGITAL I/O)
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_TASK_Main()

***********************************************************************************************/

void V_ABZB_ASP_APP_SWITCH_LED_DemoInit()
{
        #ifdef ABZB_ASP_TI_SWITCH
           V_ABZB_ASP_SRC_HardwareInit();                                 //Hardware init of SRC (Switches)
        #endif                                                            //ABZB_ASP_TI_SWITCH

        #ifdef ABZB_ASP_TI_LED
           V_ABZB_ASP_SLC_HardwareInit();                                 //Hardware init of SLC (LEDs)
        #endif                                                            //ABZB_ASP_TI_LED
}

/***********************************************************************************************
Description:	
             Make the necessary initialisations for the PAN/C to start
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_TASK_Main()

***********************************************************************************************/

void V_ABZB_APP_COORDINATOR_INIT()
{
      V_ABZB_NWK_Init(COORDINATOR,M_ABZB_STACK_PROFILE,FALSE);         //NWK init for coordinator
      V_ABZB_APP_NetworkResetRequest0();                               //calling the nwk reset request
      V_ABZB_APP_NetworkFormationRequest0();                           //calling the nwk formation request
      V_ABZB_APP_Send_NLME_PermitJoin(M_ABZB_PERMIT_DURATION);         //calling the permit join request
}

/***********************************************************************************************
Description:	
             Make the necessary initialisations for the Router to join the network
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_TASK_Main()

***********************************************************************************************/

void V_ABZB_APP_ROUTER_INIT()
{
      V_ABZB_NWK_Init(ROUTER,M_ABZB_STACK_PROFILE,FALSE);              //NWK init for ROUTER
      V_ABZB_APP_NetworkResetRequest0();                               //Calling the necessary API to make the NWK layer RESET
      V_ABZB_APP_NetworkDiscoveryRequest0();                           //Call the API to give the NLME DIscovery request
      V_ABZB_APP_NetworkJoinRequest0();                                //call the API to give the NLME Join REQUEST
      V_ABZB_APP_Network_StartRouterRequest0();                        //Call the API to give the NLME START router request
}

/***********************************************************************************************
Description:	
             Constructs a DATA REQUEST (application data)packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_Forever_Loop() when the M_DEBOUNCE_EVENT is posted
                        on the APP mailbox when the key press event has occured.

CAUTION         :       while constructing the packet the protocol ID for the application data is a must!!!

***********************************************************************************************/

void V_ABZB_APP_Send_NLDEDataRequest()
{
      stNLDEDATARequest *ABZB_Pst_DataReq;              //declaring a structure of type stNLDEDATARequest
      ABZB_Pst_DataReq = (stNLDEDATARequest *) malloc(sizeof(stNLDEDATARequest ));
      if(ABZB_Pst_DataReq != NULL_PTR)                  //checking whether the pointer is null
      {
          ABZB_Pst_DataReq-> DstAddr         = Ui_ABZB_ZADKV2_GetDstAddress();           //getting the DST NWK address from the binding table
          ABZB_Pst_DataReq-> NsduLength      = M_ABZB_APP_DATA_NSDULENGTH;               //length of the NSDU that to be sent in the air
          ABZB_Pst_DataReq-> Nsdu            =(unsigned char *)malloc(sizeof(UINT8));    //allocating memory for the NSDU
          if(ABZB_Pst_DataReq-> Nsdu != NULL_PTR)                                        //checking whether the pointer is null
          {
              *(ABZB_Pst_DataReq-> Nsdu)         = M_ABZB_PROTOCOL_ID_APP_DATA;              //NSDU is the protocol ID for application data in this case
              ABZB_Pst_DataReq-> NsduHandle      = M_ABZB_APP_DATA_NSDUHANDLE;               //Handle for the APP data (can be changed in APP_ABZB_TASK.h)
              ABZB_Pst_DataReq-> BroadcastRadius = ZERO;
              ABZB_Pst_DataReq-> DiscoverRoute   = ZERO;
              ABZB_Pst_DataReq-> SecurityEnable  = FALSE;
              V_ABZB_NWK_NLDEDATARequest(ABZB_Pst_DataReq);                                   //sending the NLDE DATA REQUEST after framing the packet
          }
      }
}


/***********************************************************************************************
Description:	
             Constructs a DATA REQUEST (permit join enable)packet and send it to the NWK layer
Function Owner  :	Airbee Wireless Inc.,
	
Return Type     :	void

Parameter Passed:       none

Called from     :	called from the V_ABZB_APP_Forever_Loop() after the reception of the two NLDE JOIN INDICATION is posted
                        on the APP mailbox in case of PAN/C in this topology.One NLME JOIN INDICATION in case of ROUTER for this topology.
                        can be chaged by user according to his requirements

CAUTION         :       while constructing the DATA REQUEST packet the protocol ID for the application data is a must!!!

***********************************************************************************************/
void V_ABZB_APP_Send_PermitEnable_To_Router()
{
      stNLMEPERMITJOININGRequest *NLME_Permit_Joining_Request;      //DECLARING A POINTER OF TYPE stNLMEPERMITJOININGRequest
      stNLDEDATARequest *ABZB_ZADKV2_Pst_DataReq;                   //DECLARING A POINTER OF TYPE stNLDEDATARequest
      NLME_Permit_Joining_Request = (stNLMEPERMITJOININGRequest*)malloc(sizeof(stNLMEPERMITJOININGRequest));
      if(NLME_Permit_Joining_Request != NULL_PTR)                   //CHECKING WHETHER THE POINTER IS NULL
      {
            NLME_Permit_Joining_Request->PermitDuration = ZERO;     //DISABLING THE PERMIT DURATION AS ZERO AS NO MORE DEVICES CAN JOIN TO THIS DEVICE
            V_ABZB_NWK_NLMEPERMITJOININGRequest(NLME_Permit_Joining_Request); //SENDING THE NLME PERMIT JOIN to the NWK layer

            ABZB_ZADKV2_Pst_DataReq = (stNLDEDATARequest *) malloc(sizeof(stNLDEDATARequest ));
            if(ABZB_ZADKV2_Pst_DataReq != NULL_PTR)                 //CHECKING WHETHER THE POINTER IS NULL
            {
                ABZB_ZADKV2_Pst_DataReq-> DstAddr = Ui_ABZB_ZADKV2_Get_Nwk_Addr();     //getting the destination address where the data to be sent

                ABZB_ZADKV2_Pst_DataReq-> NsduLength = M_ABZB_APP_DATA_NSDULENGTH;      //NSDU length (macro referred from ABZB_APP_TASK.)
                ABZB_ZADKV2_Pst_DataReq-> Nsdu = (UINT8 *)malloc(sizeof(UINT8));
                if(ABZB_ZADKV2_Pst_DataReq-> Nsdu != NULL)                              //checking whether the pointer is not NULL
                {
                  *(ABZB_ZADKV2_Pst_DataReq-> Nsdu) = M_ABZB_PROTOCOL_ID_PERMIT_JOIN;   //ADDING THE PROTOCOL ID to the NSDU as data
                    ABZB_ZADKV2_Pst_DataReq-> NsduHandle = M_ABZB_APP_DATA_NSDUHANDLE;  //appending the handle for the data packet (macro can be referred from ABZB_APP_TASK.h)
                    ABZB_ZADKV2_Pst_DataReq-> BroadcastRadius = ZERO;
                    ABZB_ZADKV2_Pst_DataReq-> DiscoverRoute = ZERO;
                    ABZB_ZADKV2_Pst_DataReq-> SecurityEnable = FALSE;
                    V_ABZB_NWK_NLDEDATARequest(ABZB_ZADKV2_Pst_DataReq);                //sending the NLDE DATA REQUEST to the NWK layer
                }
            }
      }
}

/*****************************END OF FILE****************************************************/
