/**************************************************************************************************
 *  @file ABZB_ASP_TI_SRC.c.
 *
 *  This file contains common procedures used for switch (Digital I/O)init and handling
 *

 *
 *  Copyright 2004-2005. Airbee Wireless Inc. All rights reserved.
 *	This program is protected by US and International copyright laws.

 *	Warning: This program is protected by copyright law and international
 *	treaties. Unauthorized reproduction or distribution of this program, or
 *	any portion of it, may result in severe civil or criminal penalties, and
 *	will be prosecuted to the maximum extent possible under the law.*
 *
 *
 *	Date		Author			Modifications		Version
 *  01 Jul 05	        Airbee		          NONE		          1.0
 *																
 *
* ***************************************** Header File Inclusion ********************************/
#include "ABZB_HAL_Platform.h"     // Header file for selection for Hardware platform
#include "ABZB_FOUR_NODE_APP.h"
#include "stdlib.h"                     //Standard Library used for Dynamic memory allocation

#ifdef ABZB_ASP_TI_SWITCH
/***********************************************************************************************
Description:	
This function is used to initialize the SRC hardware module which includes the PORT iinitialisation
abd configurations

Function Owner  :	Airbee Wireless
	
Return Type     :	void

Parameter Passed:       -

Called from     :	Invoked from the file APP_TASK.c on entering the Airbee_AppTask0.

***********************************************************************************************/
void V_ABZB_ASP_SRC_HardwareInit(void)
{
   M_EXTERNAL_INT_ENABLE_REG      |= (BIT4);    //SELECTING THE INT ENABLE F0R PORT1 (BIT 4,5,6)
   M_EXTERNAL_INT_EDGE_SELECT_REG |= (BIT4);    //SELECTING INTERRUPT WHEN HIGH TO LOW TRANSITION
}


/***********************************************************************************************
Description:	
This function is used when the application needs to detect the SRC functionality(key press)
on the NLDE DATA Indication).

Function Owner  :	Airbee Wireless
	
Return Type     :	UINT8 (unsigned char), which will return the key press status.

Parameter Passed:       -

Called from     :	Invoked from the file APP_TASK.c in the function V_ABZB_MAC_Main_Airbee_AppTask0()
after on getting the message NLDE_DATA_INDICATION from the APP mailbox.

***********************************************************************************************/
UINT8 Uch_ABZB_ASP_SRC_OnOffSRC(void)
{
 UINT8 ABZB_Uch_ASP_SRC_Switch = 0;
 ((M_ABZB_DSP_PORT1INPUT & BIT4) == 0)?
    (ABZB_Uch_ASP_SRC_Switch |= BIT4): (ABZB_Uch_ASP_SRC_Switch = 0);    //checking whether the key SW1 is pressed
  return ABZB_Uch_ASP_SRC_Switch ;    //if condition is TRUE then return the status as non-zero and if FALSE return 0.
}


/***********************************************************************************************
Description:	
PORT1 Interrupt Service Routine

Function Owner  :	Airbee Wireless
	
Return Type     :	-

Parameter Passed:       -

Called from     :	Called when the PORT1 interrupt is occured.A timer is invoked to check the
debounce and on expiry of the timeout event , DEBOUNCE EVENT will occur which will handle the NLDE DATA Request

***********************************************************************************************/
/* ISR name changed from PORT1 to PORT1_SRC - TP@ - 28Jun05 */

#pragma vector=PORT1_VECTOR
__interrupt void PORT1_SRC (void)
{
  P1IFG &= (~BIT4);   //CLRING THE INTERRUPT FLAG i.e., BIT4 of PORT1
  V_ABZB_Timer_Soft_set_timer3( M_DEBOUNCE_EVENT, M_ABZB_DSP_KEYDEBOUNCE, APP_MBX );    //Setting a software timer of about M_ABZB_DSP_KEYDEBOUNCE milli seconds which on expiry of that time will post the msgID
}                                                                                       //M_DEBOUNCE_EVENT IN THE APP_MBX

#endif    //ABZB_ASP_TI_SWITCH
